@@ -12,17 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-import inspect
-import re
-
-# Fix known bugs with libraries which use multi-processing in a way which conflicts with pytorch data loader
-import cv2
-
-from fastestimator import architecture, backend, dataset, layers, op, schedule, summary, trace, util, xai
-from fastestimator.estimator import Estimator, enable_deterministic, record_history
+from fastestimator import architecture, backend, dataset, layers, op, schedule, summary, trace, util, xai, search
+from fastestimator.estimator import Estimator, enable_deterministic
 from fastestimator.network import Network, build
 from fastestimator.pipeline import Pipeline
 
+# Fix known bugs with libraries which use multi-processing in a way which conflicts with pytorch data loader
+import cv2
 cv2.setNumThreads(0)
 try:
     import SimpleITK as sitk
@@ -30,15 +26,5 @@ try:
 except ModuleNotFoundError:
     pass
 
-__version__ = '1.1.5'
+__version__ = '1.2.0'
 fe_deterministic_seed = None
-fe_history_path = None  # Where to save training histories. None for ~/fastestimator_data/history.db, False to disable
-
-# Disable history logging for tests by default (they can still turn it on/off manually in setUpClass/tearDownClass)
-if __name__ != '__main__':
-    for frame in inspect.stack()[1:]:
-        if frame.filename[0] != '<':  # Filenames starting with '<' are internal
-            # frame.filename will be the name of the file which is currently importing FE
-            if re.match('.*/test/PR_test/.*test_.*\\.py', frame.filename):
-                fe_history_path = False
-            break
