@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-import inspect
 import os
 import random
 from collections import ChainMap
@@ -32,16 +31,15 @@ from fastestimator.dataset.batch_dataset import BatchDataset
 from fastestimator.network import BaseNetwork, TFNetwork, TorchNetwork
 from fastestimator.pipeline import Pipeline
 from fastestimator.schedule.schedule import Scheduler, get_current_items, get_signature_epochs
-from fastestimator.summary.history import HistoryRecorder
 from fastestimator.summary.system import Summary, System
 from fastestimator.trace.io.best_model_saver import BestModelSaver
 from fastestimator.trace.io.model_saver import ModelSaver
 from fastestimator.trace.io.restore_wizard import RestoreWizard
 from fastestimator.trace.io.traceability import Traceability
-from fastestimator.trace.trace import EvalEssential, Logger, Trace, TrainEssential, sort_traces
+from fastestimator.trace.trace import EvalEssential, Logger, TestEssential, Trace, TrainEssential, sort_traces
 from fastestimator.util.data import Data
 from fastestimator.util.traceability_util import traceable
-from fastestimator.util.util import NonContext, Suppressor, draw, to_list, to_set
+from fastestimator.util.util import Suppressor, draw, to_list, to_set
 
 
 @traceable()
@@ -68,7 +66,6 @@ class Estimator:
     monitor_names: Set[str]
     traces_in_use: List[Union[Trace, Scheduler[Trace]]]
     system: System
-    filepath: str
 
     def __init__(self,
                  pipeline: Pipeline,
@@ -80,7 +77,6 @@ class Estimator:
                  log_steps: Optional[int] = 100,
                  monitor_names: Union[None, str, Iterable[str]] = None):
         self.traces_in_use = []
-        self.filepath = os.path.realpath(inspect.stack()[2].filename)  # Record this for history tracking
         assert log_steps is None or log_steps >= 0, \
             "log_steps must be None or positive (or 0 to disable only train logging)"
         self.monitor_names = to_set(monitor_names) | network.get_loss_keys()
@@ -155,6 +151,8 @@ class Estimator:
                 print("FastEstimator-Warn: No ModelSaver Trace detected. Models will not be saved.")
         if "eval" in run_modes and "eval" in self.pipeline.get_modes():
             self.traces_in_use.insert(1, EvalEssential(monitor_names=self.monitor_names.union(extra_monitor_keys)))
+        if "test" in run_modes and "test" in self.pipeline.get_modes():
+            self.traces_in_use.insert(0, TestEssential(monitor_names=self.monitor_names.union(extra_monitor_keys)))
         # insert system instance to trace
         for trace in get_current_items(self.traces_in_use, run_modes=run_modes):
             trace.system = self.system
@@ -188,16 +186,32 @@ class Estimator:
         all_traces = get_current_items(self.traces_in_use, run_modes={"train", "eval"})
         sort_traces(all_traces)  # This ensures that the traces can sort properly for on_begin and on_end
         monitor_names = self.monitor_names
+        signature_epochs = {}
         for mode in self.pipeline.get_modes() - {"test"}:
             scheduled_items = self.pipeline.get_scheduled_items(mode) + self.network.get_scheduled_items(
                 mode) + self.get_scheduled_items(mode)
-            signature_epochs = get_signature_epochs(scheduled_items, self.system.total_epochs, mode=mode)
+            signature_epochs[mode] = get_signature_epochs(scheduled_items, self.system.total_epochs, mode=mode)
+        if max([len(x) for x in signature_epochs.values()]) < 2 and warmup != "debug":
+            return
+        for mode, signature_epochs_mode in signature_epochs.items():
             epochs_with_data = self.pipeline.get_epochs_with_data(total_epochs=self.system.total_epochs, mode=mode)
-            for epoch in signature_epochs:
+            for epoch in signature_epochs_mode:
                 if epoch not in epochs_with_data:
                     continue
+                network_output_keys = self.network.get_all_output_keys(mode, epoch)
+                network_input_keys = self.network.get_effective_input_keys(mode, epoch)
+                trace_input_keys = set()
+                trace_output_keys = {"*"}
+                traces = get_current_items(self.traces_in_use, run_modes=mode, epoch=epoch)
+                for idx, trace in enumerate(traces):
+                    if idx > 0:  # ignore TrainEssential and EvalEssential's inputs for unmet requirement checking
+                        trace_input_keys.update(trace.inputs)
+                    trace_output_keys.update(trace.outputs)
                 # key checking
-                loader = self._configure_loader(self.pipeline.get_loader(mode, epoch))
+                loader = self._configure_loader(
+                    self.pipeline.get_loader(mode,
+                                             epoch,
+                                             output_keys=trace_input_keys - network_output_keys | network_input_keys))
                 with Suppressor():
                     if isinstance(loader, tf.data.Dataset):
                         batch = list(loader.take(1))[0]
@@ -206,14 +220,7 @@ class Estimator:
                 batch = self._configure_tensor(loader, batch)
                 assert isinstance(batch, dict), "please make sure data output format is dictionary"
                 pipeline_output_keys = to_set(batch.keys())
-                network_output_keys = self.network.get_all_output_keys(mode, epoch)
-                trace_input_keys = set()
-                trace_output_keys = {"*"}
-                traces = get_current_items(self.traces_in_use, run_modes=mode, epoch=epoch)
-                for idx, trace in enumerate(traces):
-                    if idx > 0:  # ignore TrainEssential and EvalEssential's inputs for unmet requirement checking
-                        trace_input_keys.update(trace.inputs)
-                    trace_output_keys.update(trace.outputs)
+
                 monitor_names = monitor_names - (pipeline_output_keys | network_output_keys)
                 unmet_requirements = trace_input_keys - (pipeline_output_keys | network_output_keys | trace_output_keys)
                 assert not unmet_requirements, \
@@ -246,27 +253,25 @@ class Estimator:
             run_modes: The current execution modes.
         """
         all_traces = sort_traces(get_current_items(self.traces_in_use, run_modes=run_modes))
-        with NonContext() if fe.fe_history_path is False else HistoryRecorder(
-                self.system, self.filepath, db_path=fe.fe_history_path):
-            try:
-                self._run_traces_on_begin(traces=all_traces)
-                if "train" in run_modes or "eval" in run_modes:
-                    # If the training is re-starting from a restore wizard, it should re-run the last eval epoch
-                    if self.system.epoch_idx > 0 and "eval" in self.pipeline.get_modes(epoch=self.system.epoch_idx):
+        try:
+            self._run_traces_on_begin(traces=all_traces)
+            if "train" in run_modes or "eval" in run_modes:
+                # If the training is re-starting from a restore wizard, it should re-run the last eval epoch
+                if self.system.epoch_idx > 0 and "eval" in self.pipeline.get_modes(epoch=self.system.epoch_idx):
+                    self.system.mode = "eval"
+                    self._run_epoch()
+                for self.system.epoch_idx in range(self.system.epoch_idx + 1, self.system.total_epochs + 1):
+                    if "train" in self.pipeline.get_modes(epoch=self.system.epoch_idx):
+                        self.system.mode = "train"
+                        self._run_epoch()
+                    if "eval" in self.pipeline.get_modes(epoch=self.system.epoch_idx):
                         self.system.mode = "eval"
                         self._run_epoch()
-                    for self.system.epoch_idx in range(self.system.epoch_idx + 1, self.system.total_epochs + 1):
-                        if "train" in self.pipeline.get_modes(epoch=self.system.epoch_idx):
-                            self.system.mode = "train"
-                            self._run_epoch()
-                        if "eval" in self.pipeline.get_modes(epoch=self.system.epoch_idx):
-                            self.system.mode = "eval"
-                            self._run_epoch()
-                else:
-                    self._run_epoch()
-            except EarlyStop:
-                pass  # On early stopping we still want to run the final traces and return results
-            self._run_traces_on_end(traces=all_traces)
+            else:
+                self._run_epoch()
+        except EarlyStop:
+            pass  # On early stopping we still want to run the final traces and return results
+        self._run_traces_on_end(traces=all_traces)
 
     def _run_epoch(self) -> None:
         """A method to perform an epoch of activity.
@@ -277,16 +282,18 @@ class Estimator:
         trace_input_keys = set()
         for trace in traces:
             trace_input_keys.update(trace.inputs)
-        loader = self._configure_loader(self.pipeline.get_loader(self.system.mode, self.system.epoch_idx))
+        network_input_keys = self.network.get_effective_input_keys(self.system.mode, self.system.epoch_idx)
+        network_output_keys = self.network.get_all_output_keys(self.system.mode, self.system.epoch_idx)
+        loader = self._configure_loader(
+            self.pipeline.get_loader(self.system.mode,
+                                     self.system.epoch_idx,
+                                     output_keys=trace_input_keys - network_output_keys | network_input_keys))
         iterator = iter(loader)
         self.network.load_epoch(mode=self.system.mode, epoch=self.system.epoch_idx, output_keys=trace_input_keys)
         self.system.batch_idx = None
         with Suppressor():
             batch = next(iterator)
-        traces = sort_traces(
-            traces,
-            available_outputs=to_set(batch.keys())
-            | self.network.get_all_output_keys(self.system.mode, self.system.epoch_idx))
+        traces = sort_traces(traces, available_outputs=to_set(batch.keys()) | network_output_keys)
         self._run_traces_on_epoch_begin(traces=traces)
         while True:
             try:
@@ -335,8 +342,12 @@ class Estimator:
                 new_loader = new_loader.take(self.system.max_train_steps_per_epoch)
             if self.system.max_eval_steps_per_epoch and self.system.mode == "eval":
                 new_loader = new_loader.take(self.system.max_eval_steps_per_epoch)
-            if isinstance(tf.distribute.get_strategy(),
-                          tf.distribute.MirroredStrategy) and not isinstance(new_loader, DistributedDataset):
+            if isinstance(tf.distribute.get_strategy(), tf.distribute.MirroredStrategy) and isinstance(
+                    self.network, TFNetwork) and not isinstance(new_loader, DistributedDataset):
+                # The default autoshard policy is file, changing it to data to avoid warning
+                options = tf.data.Options()
+                options.experimental_distribute.auto_shard_policy = tf.data.experimental.AutoShardPolicy.DATA
+                new_loader = new_loader.with_options(options)
                 new_loader = tf.distribute.get_strategy().experimental_distribute_dataset(new_loader)
         return new_loader
 
@@ -455,16 +466,13 @@ class EarlyStop(Exception):
     """
 
 
-def enable_deterministic(seed: int) -> None:
-    """Invoke to set random seed for deterministic training.
-
-    The determinism only works for tensorflow >= 2.1 and pytorch >= 1.14, and some model layers don't support.
+def enable_deterministic(seed):
+    """Invoke to set random seed for deterministic training. The determinism only works for tensorflow >= 2.1 and
+    pytorch >= 1.14, and some model layers don't support.
 
     Known failing layers:
     * tf.keras.layers.UpSampling2D
 
-    Args:
-        seed: The random seed to use for training.
     """
     fe.fe_deterministic_seed = seed
     os.environ['PYTHONHASHSEED'] = str(seed)
@@ -473,16 +481,3 @@ def enable_deterministic(seed: int) -> None:
     np.random.seed(seed)
     tf.random.set_seed(seed)
     torch.manual_seed(seed)
-
-
-def record_history(path: Union[bool, str]) -> None:
-    """Change the default location for history tracking.
-
-    Args:
-        path: The path to save experiment histories. Pass True to use the default location of
-            ~/fastestimator_data/history.db. Pass False to disable history tracking.
-    """
-    if path in (None, True):
-        fe.fe_history_path = None
-    else:
-        fe.fe_history_path = path
