@@ -18,16 +18,18 @@ from collections import ChainMap
 from typing import Any, Callable, Dict, Iterable, List, MutableMapping, Optional, Set, Tuple, TypeVar, Union
 
 import gdown
+import numpy as np
 import tensorflow as tf
+import tensorflow.keras.mixed_precision as mixed_precision_tf
 import torch
-from tensorflow.keras.mixed_precision import experimental as mixed_precision_tf
 from tensorflow.python.distribute.values import DistributedValues
 
 from fastestimator.backend.load_model import load_model
 from fastestimator.backend.to_tensor import to_tensor
+from fastestimator.op.numpyop import NumpyOp, forward_numpyop
 from fastestimator.op.op import get_inputs_by_op, write_outputs_by_op
-from fastestimator.op.tensorop import TensorOp
-from fastestimator.op.tensorop.model import UpdateOp
+from fastestimator.op.tensorop.model.update import UpdateOp
+from fastestimator.op.tensorop.tensorop import TensorOp
 from fastestimator.schedule.schedule import EpochScheduler, RepeatScheduler, Scheduler, get_current_items
 from fastestimator.util.traceability_util import trace_model, traceable
 from fastestimator.util.util import NonContext, get_batch_size, to_list
@@ -45,19 +47,40 @@ class BaseNetwork:
     Networks are used to define the computation graph surrounding one or more models during training.
 
     Args:
+        target_type: What tensor type is expected by this network ('torch' or 'tf').
         ops: The operators to be executed throughout training / testing / inference. These are likely to contain one or
             more model ops, as well as loss ops and update ops.
+        postprocessing: A collection of NumpyOps to be run on the CPU after all of the normal `ops` have been executed.
+            Unlike the NumpyOps found in the pipeline, these ops will run on batches of data rather than single points.
+
+    Raises:
+        ValueError: Mixed precision settings for all models are not the same.
     """
-    def __init__(self, ops: Iterable[Union[TensorOp, Scheduler[TensorOp]]]) -> None:
+    def __init__(
+        self,
+        target_type: str,
+        device: Optional[torch.device],
+        ops: Iterable[Union[TensorOp, Scheduler[TensorOp]]],
+        postprocessing: Union[None, NumpyOp, Scheduler[NumpyOp], Iterable[Union[NumpyOp, Scheduler[NumpyOp]]]] = None
+    ) -> None:
         self.ops = to_list(ops)
+        self.target_type = target_type
+        self.device = device
+        for op in get_current_items(self.ops):
+            op.build(framework=self.target_type, device=self.device)
         self.models = to_list(_collect_models(ops))
+        self.postprocessing = to_list(postprocessing)
         self._verify_inputs()
         self.effective_inputs = dict()
         self.effective_outputs = dict()
         self.epoch_ops = []
+        self.epoch_postprocessing = []
         self.epoch_models = set()
         self.epoch_state = dict()
-        self.scaler = None
+        self.mixed_precision = any([model.mixed_precision for model in self.models])
+
+        if self.mixed_precision and not all([model.mixed_precision for model in self.models]):
+            raise ValueError("Cannot mix full precision and mixed-precision models")
 
     def _verify_inputs(self) -> None:
         """Ensure that all ops are TensorOps.
@@ -66,7 +89,9 @@ class BaseNetwork:
             AssertionError: If any of the ops are not TensorOps.
         """
         for op in get_current_items(self.ops):
-            assert isinstance(op, TensorOp), "unsupported op format, must provide TensorOp in Network"
+            assert isinstance(op, TensorOp), "unsupported op format, Network ops must be TensorOps"
+        for op in get_current_items(self.postprocessing):
+            assert isinstance(op, NumpyOp), "unsupported op format, Network postprocessing must be NumpyOps"
 
     def get_scheduled_items(self, mode: str) -> List[Any]:
         """Get a list of items considered for scheduling.
@@ -78,9 +103,9 @@ class BaseNetwork:
             List of schedulable items in Network.
         """
         if mode == "train":
-            all_items = self.ops + [model.optimizer for model in self.models]
+            all_items = self.ops + [model.optimizer for model in self.models] + self.postprocessing
         else:
-            all_items = self.ops
+            all_items = self.ops + self.postprocessing
         return all_items
 
     def load_epoch(self, mode: str, epoch: int, output_keys: Optional[Set[str]] = None, warmup: bool = False) -> None:
@@ -92,25 +117,22 @@ class BaseNetwork:
         Args:
             mode: The mode to prepare to execute. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch to prepare to execute.
-            output_keys: What keys must be moved from the GPU back to the CPU after executing a step.
+            output_keys: What keys can be moved from the GPU back to the CPU after executing a step.
             warmup: Whether to prepare to execute it warmup mode or not (end users can likely ignore this argument).
         """
         self.effective_inputs[mode] = self.get_effective_input_keys(mode, epoch)
         self.effective_outputs[mode] = self.get_all_output_keys(mode, epoch)
         if output_keys:
-            self.effective_outputs[mode] = self.effective_outputs[mode].intersection(output_keys)
+            self.effective_outputs[mode] = self.effective_outputs[mode].intersection(
+                output_keys) | self._get_effective_postprocessing_input_keys(mode, epoch)
         self.epoch_ops = get_current_items(self.ops, mode, epoch)
+        self.epoch_postprocessing = get_current_items(self.postprocessing, mode, epoch)
         self.epoch_models = set.union(*[op.get_fe_models() for op in self.epoch_ops])
         gradient_ops = [op for op in self.epoch_ops if op.fe_retain_graph() is not None]
         for idx, gradient_op in enumerate(gradient_ops):
             gradient_op.fe_retain_graph(idx != len(gradient_ops) - 1)
         self.epoch_state = {
-            "warmup": warmup,
-            "mode": mode,
-            "req_grad": len(gradient_ops) > 0,
-            "epoch": epoch,
-            "deferred": {},
-            "scaler": self.scaler
+            "warmup": warmup, "mode": mode, "req_grad": len(gradient_ops) > 0, "epoch": epoch, "deferred": {}
         }
         # warmup: bool, mode: str, req_grad: bool, epoch: int, deferred: Dict[str, List[Callable]]]
         for model in self.epoch_models:
@@ -148,7 +170,24 @@ class BaseNetwork:
         """
         input_keys = set()
         produced_keys = set()
-        for op in get_current_items(self.ops, mode, epoch):
+        for op in get_current_items(self.ops + self.postprocessing, mode, epoch):
+            input_keys.update(set(key for key in op.inputs if key not in produced_keys))
+            produced_keys.update(op.outputs)
+        return input_keys
+
+    def _get_effective_postprocessing_input_keys(self, mode: str, epoch: int) -> Set[str]:
+        """Determine which keys need to be provided as input to the postprocessing during the given `epoch`.
+
+        Args:
+            mode: The execution mode to consider. One of 'train', 'eval', 'test', or 'infer'.
+            epoch: The epoch number to consider for determining inputs.
+
+        Returns:
+            The necessary inputs for the postprocessing to execute the given `epoch` and `mode`.
+        """
+        input_keys = set()
+        produced_keys = set()
+        for op in get_current_items(self.postprocessing, mode, epoch):
             input_keys.update(set(key for key in op.inputs if key not in produced_keys))
             produced_keys.update(op.outputs)
         return input_keys
@@ -164,7 +203,7 @@ class BaseNetwork:
             The keys that will be generated by the network's Ops during the `epoch` for the given `mode`.
         """
         output_keys = set()
-        for op in get_current_items(self.ops, mode, epoch):
+        for op in get_current_items(self.ops + self.postprocessing, mode, epoch):
             output_keys.update(op.outputs)
         return output_keys
 
@@ -189,7 +228,25 @@ class BaseNetwork:
         state['deferred'].clear()
 
     def run_step(self, batch: Dict[str, Any]) -> Tuple[Dict[str, Any], Dict[str, Any]]:  # Batch, Prediction
-        """Run a forward step through the Network on a batch of data.
+        """Run a forward step through the Network on a batch of data, including postprocessing.
+
+        This method expects that Network.load_epoch() has already been invoked. The return data will be on the CPU.
+
+        Args:
+            batch: The batch of data serving as input to the Network.
+
+        Returns:
+            (batch_data, prediction_data)
+        """
+        batch, prediction = self._run_step(batch)
+        forward_numpyop(ops=self.epoch_postprocessing,
+                        data=ChainMap(prediction, batch),
+                        state=self.epoch_state,
+                        batched=True)
+        return batch, prediction
+
+    def _run_step(self, batch: Dict[str, Any]) -> Tuple[Dict[str, Any], Dict[str, Any]]:  # Batch, Prediction
+        """Run a forward step through the Network on a batch of data, excluding postprocessing.
 
         Implementations of this method within derived classes should handle bringing the prediction data back from the
         (multi-)GPU environment to the CPU. This method expects that Network.load_epoch() has already been invoked.
@@ -211,9 +268,13 @@ class BaseNetwork:
             epoch: The epoch in which to run the transform.
 
         Returns:
-            (batch_data, prediction_data)
+            prediction_data overlaid on the input `data`.
         """
-        raise NotImplementedError
+        self.load_epoch(mode, epoch, warmup=False)
+        data = to_tensor(data, target_type=self.target_type)
+        data, prediction = self.run_step(data)
+        self.unload_epoch()
+        return {**data, **prediction}
 
 
 def _collect_models(ops: Iterable[Union[TensorOp, Scheduler[TensorOp]]]) -> Set[Model]:
@@ -232,13 +293,19 @@ def _collect_models(ops: Iterable[Union[TensorOp, Scheduler[TensorOp]]]) -> Set[
 
 
 # noinspection PyPep8Naming
-def Network(ops: Iterable[Union[TensorOp, Scheduler[TensorOp]]]) -> BaseNetwork:
+def Network(
+        ops: Iterable[Union[TensorOp, Scheduler[TensorOp]]],
+        pops: Union[None, NumpyOp, Scheduler[NumpyOp], Iterable[Union[NumpyOp,
+                                                                      Scheduler[NumpyOp]]]] = None) -> BaseNetwork:
     """A function to automatically instantiate the correct Network derived class based on the given `ops`.
 
     Args:
         ops: A collection of Ops defining the graph for this Network. It should contain at least one ModelOp, and all
             models should be either TensorFlow or Pytorch. We currently do not support mixing TensorFlow and Pytorch
             models within the same network.
+        pops: Postprocessing Ops. A collection of NumpyOps to be run on the CPU after all of the normal `ops` have been
+            executed. Unlike the NumpyOps found in the pipeline, these ops will run on batches of data rather than
+            single points.
 
     Returns:
         A network instance containing the given `ops`.
@@ -266,9 +333,9 @@ def Network(ops: Iterable[Union[TensorOp, Scheduler[TensorOp]]]) -> BaseNetwork:
 
     framework = framework.pop()
     if framework == "tf":
-        network = TFNetwork(ops)
+        network = TFNetwork(ops, pops)
     elif framework == "torch":
-        network = TorchNetwork(ops)
+        network = TorchNetwork(ops, pops)
     else:
         raise ValueError("Unknown model type")
     return network
@@ -280,14 +347,19 @@ class TorchNetwork(BaseNetwork):
 
     Args:
         ops: The ops defining the execution graph for this Network.
+        postprocessing: A collection of NumpyOps to be run on the CPU after all of the normal `ops` have been executed.
+            Unlike the NumpyOps found in the pipeline, these ops will run on batches of data rather than single points.
+
     """
-    def __init__(self, ops: Iterable[Union[TensorOp, Scheduler[TensorOp]]]) -> None:
-        super().__init__(ops)
-        for op in get_current_items(self.ops):
-            op.build(framework='torch')
-        self.device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
-        if any([model.mixed_precision for model in self.models]):
-            self.scaler = torch.cuda.amp.GradScaler()
+    def __init__(
+        self,
+        ops: Iterable[Union[TensorOp, Scheduler[TensorOp]]],
+        postprocessing: Union[None, NumpyOp, Scheduler[NumpyOp], Iterable[Union[NumpyOp, Scheduler[NumpyOp]]]] = None
+    ) -> None:
+        super().__init__(target_type='torch',
+                         device=torch.device("cuda:0" if torch.cuda.is_available() else "cpu"),
+                         ops=ops,
+                         postprocessing=postprocessing)
 
     def load_epoch(self, mode: str, epoch: int, output_keys: Optional[Set[str]] = None, warmup: bool = False) -> None:
         """Prepare the network to run a given epoch and mode.
@@ -375,7 +447,7 @@ class TorchNetwork(BaseNetwork):
             new_batch = {key: batch[key] for key in self.effective_inputs[mode] if key in batch}
         return new_batch
 
-    def run_step(self, batch: Dict[str, Any]) -> Tuple[Dict[str, Any], Dict[str, Any]]:
+    def _run_step(self, batch: Dict[str, Any]) -> Tuple[Dict[str, Any], Dict[str, Any]]:
         """Run a forward step through the Network on a batch of data.
 
         Implementations of this method within derived classes should handle bringing the prediction data back from the
@@ -392,12 +464,9 @@ class TorchNetwork(BaseNetwork):
         self.epoch_state["tape"] = NonContext()
         # gpu operation
         with torch.no_grad() if not self.epoch_state["req_grad"] else NonContext():
-            with torch.cuda.amp.autocast() if self.epoch_state["scaler"] is not None else NonContext():
+            with torch.cuda.amp.autocast() if self.mixed_precision else NonContext():
                 self._forward_batch(batch_in, self.epoch_state, self.epoch_ops)
-        # if the loss scaler is used for training, update the scaler
-        if not self.epoch_state["warmup"] and self.epoch_state[
-                "mode"] == "train" and self.epoch_state["scaler"] is not None:
-            self.epoch_state["scaler"].update()
+
         # copy data to cpu
         if self.device.type == "cuda":
             prediction = {
@@ -453,23 +522,6 @@ class TorchNetwork(BaseNetwork):
             return set([self._detach_tensor(val) for val in data])
         elif isinstance(data, torch.Tensor):
             return data.detach()
-
-    def transform(self, data: Dict[str, Any], mode: str, epoch: int = 1) -> Dict[str, Any]:
-        """Run a forward step through the Network on an element of data.
-
-        Args:
-            data: The element to data to use as input.
-            mode: The mode in which to run the transform. One of 'train', 'eval', 'test', or 'infer'.
-            epoch: The epoch in which to run the transform.
-
-        Returns:
-            (batch_data, prediction_data)
-        """
-        self.load_epoch(mode, epoch, warmup=False)
-        data = to_tensor(data, "torch")
-        data, prediction = self.run_step(data)
-        self.unload_epoch()
-        data.update(prediction)
         return data
 
 
@@ -479,11 +531,15 @@ class TFNetwork(BaseNetwork):
 
     Args:
         ops: The ops defining the execution graph for this Network.
+        postprocessing: A collection of NumpyOps to be run on the CPU after all of the normal `ops` have been executed.
+            Unlike the NumpyOps found in the pipeline, these ops will run on batches of data rather than single points.
     """
-    def __init__(self, ops: Iterable[Union[TensorOp, Scheduler[TensorOp]]]) -> None:
-        super().__init__(ops)
-        for op in get_current_items(self.ops):
-            op.build(framework='tf')
+    def __init__(
+        self,
+        ops: Iterable[Union[TensorOp, Scheduler[TensorOp]]],
+        postprocessing: Union[None, NumpyOp, Scheduler[NumpyOp], Iterable[Union[NumpyOp, Scheduler[NumpyOp]]]] = None
+    ) -> None:
+        super().__init__(target_type='tf', device=None, ops=ops, postprocessing=postprocessing)
 
     def load_epoch(self, mode: str, epoch: int, output_keys: Optional[Set[str]] = None, warmup: bool = False) -> None:
         """Prepare the network to run a given epoch and mode.
@@ -498,9 +554,14 @@ class TFNetwork(BaseNetwork):
             warmup: Whether to prepare to execute it warmup mode or not (end users can likely ignore this argument).
         """
         super().load_epoch(mode, epoch, output_keys, warmup)
+        # Don't cause a re-trace just because epoch changed
         self.epoch_state["epoch"] = tf.convert_to_tensor(self.epoch_state["epoch"])
+        # Need to re-trace the TF graph if the optimizer is changing due to scheduling:
+        opt_str = "x".join(
+            [str(id(model.current_optimizer)) for model in self.epoch_models if hasattr(model, 'current_optimizer')])
+        self.epoch_state["_force_tf_retrace"] = hash(opt_str)  # Hash to keep at fixed memory overhead
 
-    def run_step(self, batch: Dict[str, Any]) -> Tuple[Dict[str, Any], Dict[str, Any]]:
+    def _run_step(self, batch: Dict[str, Any]) -> Tuple[Dict[str, Any], Dict[str, Any]]:
         """Run a forward step through the Network on a batch of data.
 
         Implementations of this method within derived classes should handle bringing the prediction data back from the
@@ -656,21 +717,50 @@ class TFNetwork(BaseNetwork):
         Returns:
             (batch_data, prediction_data)
         """
-        self.load_epoch(mode, epoch, warmup=False)
-        data = to_tensor(data, target_type="tf")
-        data, prediction = self.run_step(data)
-        self.unload_epoch()
-        # handle tensorflow multi-gpu inferencing issue, it will replicate data on each device
-        if isinstance(tf.distribute.get_strategy(), tf.distribute.MirroredStrategy):
-            prediction = self._subsample_data(prediction, get_batch_size(data))
-        data.update(prediction)
-        return data
+        # Distribute multi-gpu data for processing
+        sub_sample = False
+        strategy = tf.distribute.get_strategy()
+        if isinstance(strategy, tf.distribute.MirroredStrategy):
+            batch_size, num_devices = get_batch_size(data), strategy.num_replicas_in_sync
+            if batch_size < num_devices:
+                data = self._fill_batch(data, num_devices - batch_size)
+                sub_sample = True
+            data = next(iter(strategy.experimental_distribute_dataset(tf.data.Dataset.from_tensors(data))))
+        results = super().transform(data, mode, epoch)
+        if sub_sample:
+            results = self._subsample_data(results, batch_size)
+        return results
+
+    def _fill_batch(self, data: T, n: int) -> T:
+        """Fill data on batch dimension repeating the first n indices at the end.
+
+        Args:
+            data: The data to be filled.
+            n: The number of times to be repeated.
+
+        Returns:
+            Filled data.
+        """
+        if isinstance(data, dict):
+            return {key: self._fill_batch(val, n) for (key, val) in data.items()}
+        elif isinstance(data, list):
+            return [self._fill_batch(val, n) for val in data]
+        elif isinstance(data, tuple):
+            return tuple([self._fill_batch(val, n) for val in data])
+        elif isinstance(data, set):
+            return set([self._fill_batch(val, n) for val in data])
+        elif hasattr(data, "shape"):
+            paddings = [[0, n]] + [[0, 0] for _ in range(len(data.shape) - 1)]
+            return np.pad(data, pad_width=paddings, mode="symmetric")
+        else:
+            return data
 
     def _subsample_data(self, data: T, n: int) -> T:
         """Subsample data by selecting the first n indices recursively.
 
         Args:
             data: The data to be subsampled.
+            n: The number of indices to be subsampled.
 
         Returns:
             Subsampled data.
@@ -720,7 +810,7 @@ def build(model_fn: Callable[[], Union[Model, List[Model]]],
             automatically generated and assigned.
         weights_path: Path(s) from which to load model weights. If not None, then the number of weight paths provided
             should match the number of models generated by the `model_fn`.
-        mixed_precision: Whether to enable mix precision network operations.
+        mixed_precision: Whether to enable mixed-precision network operations.
 
     Returns:
         models: The model(s) built by FastEstimator.
@@ -732,12 +822,6 @@ def build(model_fn: Callable[[], Union[Model, List[Model]]],
 
     if not hasattr(build, "count"):
         build.count = 0
-    # Tensorflow models require setting global policies prior to model creation. Since there is no way to know the
-    # framework of the model ahead of time, set the policy for both tf and pytorch here.
-    if mixed_precision:
-        mixed_precision_tf.set_policy(mixed_precision_tf.Policy('mixed_float16'))
-    else:
-        mixed_precision_tf.set_policy(mixed_precision_tf.Policy('float32'))
     models, optimizer_fn = to_list(model_fn()), to_list(optimizer_fn)
     # fill optimizer
     if not optimizer_fn:
@@ -745,6 +829,11 @@ def build(model_fn: Callable[[], Union[Model, List[Model]]],
     # check framework
     if isinstance(models[0], tf.keras.Model):
         framework = "tf"
+        # tensorflow mix-precision instantiation
+        if mixed_precision:
+            mixed_precision_tf.set_global_policy(mixed_precision_tf.Policy('mixed_float16'))
+        else:
+            mixed_precision_tf.set_global_policy(mixed_precision_tf.Policy('float32'))
     elif isinstance(models[0], torch.nn.Module):
         framework = "torch"
     else:
@@ -772,7 +861,7 @@ def build(model_fn: Callable[[], Union[Model, List[Model]]],
         "Found inconsistency in number of models, optimizers, model_name or weights"
     # create optimizer
     for idx, (model, optimizer_def, weight, name) in enumerate(zip(models, optimizer_fn, weights_path, model_name)):
-        models[idx] = trace_model(_fe_compile(model, optimizer_def, weight, name, framework),
+        models[idx] = trace_model(_fe_compile(model, optimizer_def, weight, name, framework, mixed_precision),
                                   model_idx=idx if len(models) > 1 else -1,
                                   model_fn=model_fn,
                                   optimizer_fn=optimizer_def,
@@ -786,7 +875,8 @@ def _fe_compile(model: Model,
                 optimizer_fn: Union[str, Scheduler, Callable, None],
                 weight: Union[str, None],
                 name: str,
-                framework: str) -> Model:
+                framework: str,
+                mixed_precision: bool) -> Model:
     """A function to bundle models with their optimizers.
 
     Args:
@@ -795,6 +885,7 @@ def _fe_compile(model: Model,
         weight: A path to weights to be associated with the `model`.
         name: The name of the model.
         framework: Which backend framework should be associated with this model (either 'tf' or 'torch').
+        mixed_precision: Whether to enable mixed-precision training.
 
     Returns:
         The `model` combined with its optimizer, weights, and name. Models will also have an 'fe_compiled' flag to
@@ -802,15 +893,17 @@ def _fe_compile(model: Model,
     """
     if isinstance(optimizer_fn, EpochScheduler):
         for epoch, optimizer_def in optimizer_fn.epoch_dict.items():
-            optimizer_fn.epoch_dict[epoch] = _build_optimizer(optimizer_def, model, framework)
+            optimizer_fn.epoch_dict[epoch] = _build_optimizer(optimizer_def, model, framework, mixed_precision)
         model.current_optimizer = optimizer_fn.get_current_value(1)
     elif isinstance(optimizer_fn, RepeatScheduler):
         for idx, optimizer_def in enumerate(optimizer_fn.repeat_list):
-            optimizer_fn.repeat_list[idx] = _build_optimizer(optimizer_def, model, framework)
+            optimizer_fn.repeat_list[idx] = _build_optimizer(optimizer_def, model, framework, mixed_precision)
         model.current_optimizer = optimizer_fn.get_current_value(1)
     else:
-        optimizer_fn = _build_optimizer(optimizer_fn, model, framework)
+        optimizer_fn = _build_optimizer(optimizer_fn, model, framework, mixed_precision)
         model.current_optimizer = optimizer_fn
+    if framework == "tf":
+        model.input_spec = None  # this is to handle a behavior change in tf 2.4.1 that enforces a input shape check
     model.optimizer = optimizer_fn
     model.fe_compiled = True
 
@@ -827,8 +920,8 @@ def _fe_compile(model: Model,
     return model
 
 
-def _build_optimizer(optimizer_fn: Union[str, Callable, None], model: Model,
-                     framework: str) -> Union[None, tf.optimizers.Optimizer, torch.optim.Optimizer]:
+def _build_optimizer(optimizer_fn: Union[str, Callable, None], model: Model, framework: str,
+                     mixed_precision: bool) -> Union[None, tf.optimizers.Optimizer, torch.optim.Optimizer]:
     """A helper method to instantiate an optimizer.
 
     Args:
@@ -836,13 +929,14 @@ def _build_optimizer(optimizer_fn: Union[str, Callable, None], model: Model,
             default optimizers to be used.
         model: The model to associate the optimizer with.
         framework: Which backend framework should be used ('tf' or 'torch').
+        mixed_precision: Whether to enable mixed-precision training.
 
     Returns:
         An optimizer instance, or None if `optimizer_fn` was None.
     """
     if isinstance(optimizer_fn, str):
         optimizer_fn = _optimizer_fn_from_string(optimizer_fn, framework)
-    optimizer = _optimizer_fn_to_optimizer(optimizer_fn, model, framework)
+    optimizer = _optimizer_fn_to_optimizer(optimizer_fn, model, framework, mixed_precision)
     return optimizer
 
 
@@ -879,14 +973,15 @@ def _optimizer_fn_from_string(name: str, framework: str) -> Callable:
     return optimizer_fn
 
 
-def _optimizer_fn_to_optimizer(optimizer_fn: Union[Callable, None], model: Model,
-                               framework: str) -> Union[None, tf.optimizers.Optimizer, torch.optim.Optimizer]:
+def _optimizer_fn_to_optimizer(optimizer_fn: Union[Callable, None], model: Model, framework: str,
+                               mixed_precision: bool) -> Union[None, tf.optimizers.Optimizer, torch.optim.Optimizer]:
     """A helper function to invoke an optimizer function.
 
     Args:
         optimizer_fn: The function to be invoked in order to instantiate an optimizer.
         model: The model with which the optimizer should be associated.
         framework: Which backend framework should be used ('tf' or 'torch').
+        mixed_precision: Whether to enable mixed-precision training.
 
     Returns:
         An optimizer instance, or None if `optimizer_fn` was None.
@@ -899,14 +994,13 @@ def _optimizer_fn_to_optimizer(optimizer_fn: Union[Callable, None], model: Model
             except:
                 raise AssertionError("optimizer_fn of Tensorflow backend should be callable without args. Please "
                                      "make sure model and optimizer_fn are using the same backend")
-
             # initialize optimizer variables
             _ = optimizer.iterations
             optimizer._create_hypers()
             optimizer._create_slots(model.trainable_variables)
             # handle mixed precision loss scaling
-            if mixed_precision_tf.global_policy().name != "float32":
-                optimizer = mixed_precision_tf.LossScaleOptimizer(optimizer, loss_scale='dynamic')
+            if mixed_precision:
+                optimizer = mixed_precision_tf.LossScaleOptimizer(optimizer)
             assert isinstance(optimizer, tf.optimizers.Optimizer), "optimizer_fn should generate tensorflow optimizer"
         else:
             try:
@@ -916,4 +1010,9 @@ def _optimizer_fn_to_optimizer(optimizer_fn: Union[Callable, None], model: Model
                 optimizer_fn are using the same backend")
                 raise ValueError(repr(e))
             assert isinstance(optimizer, torch.optim.Optimizer), "optimizer_fn should generate pytorch optimizer"
+            if mixed_precision:
+                setattr(optimizer, "scaler", torch.cuda.amp.GradScaler())
+            else:
+                setattr(optimizer, "scaler", None)
+
     return optimizer
