@@ -32,7 +32,7 @@ from fastestimator.op.numpyop.meta.sometimes import Sometimes
 from fastestimator.op.numpyop.numpyop import NumpyOp, forward_numpyop
 from fastestimator.schedule.schedule import Scheduler, get_current_items
 from fastestimator.util.traceability_util import traceable
-from fastestimator.util.util import get_num_devices, pad_batch, to_list, to_set
+from fastestimator.util.util import pad_batch, to_list, to_set
 
 DataSource = TypeVar('DataSource', Dataset, DataLoader, tf.data.Dataset)
 
@@ -50,8 +50,8 @@ class Pipeline:
         ops: NumpyOps to be used for pre-processing. NOTE: This argument is only applicable when using a FastEstimator
             Dataset.
         num_process: Number of CPU threads to use for data pre-processing. NOTE: This argument is only applicable when
-            using a FastEstimator Dataset. None will default to min(n_cpus, max(32, 32*n_gpus)). Multiprocessing can be
-            disabled by passing 0 here, which can be useful for debugging.
+            using a FastEstimator Dataset. None will default to the system CPU count. Multiprocessing can be disabled by
+            passing 0 here, which can be useful for debugging.
         drop_last: Whether to drop the last batch if the last batch is incomplete.
         pad_value: The padding value if batch padding is needed. None indicates that no padding is needed. NOTE: This
             argument is only applicable when using a FastEstimator Dataset.
@@ -63,7 +63,7 @@ class Pipeline:
                  train_data: Union[None, DataSource, Scheduler[DataSource]] = None,
                  eval_data: Union[None, DataSource, Scheduler[DataSource]] = None,
                  test_data: Union[None, DataSource, Scheduler[DataSource]] = None,
-                 batch_size: Union[None, int, Scheduler[int]] = None,
+                 batch_size: Union[None, int, Scheduler[Union[int, Dict[str, int]]], Dict[str, int]] = None,
                  ops: Union[None, NumpyOp, Scheduler[NumpyOp], List[Union[NumpyOp, Scheduler[NumpyOp]]]] = None,
                  num_process: Optional[int] = None,
                  drop_last: bool = False,
@@ -77,7 +77,7 @@ class Pipeline:
         if mp.get_start_method(allow_none=True) != 'fork':
             print("FastEstimator-Warn: Pipeline multiprocessing is disabled. OS must support the 'fork' start method.")
             num_process = 0
-        self.num_process = num_process if num_process is not None else min(os.cpu_count(), 32 * get_num_devices())
+        self.num_process = num_process if num_process is not None else os.cpu_count()
         self.drop_last = drop_last
         self.pad_value = pad_value
         self.collate_fn = collate_fn
@@ -118,7 +118,12 @@ class Pipeline:
         if isinstance(dataset, Dataset):
             # batch_size check
             for batch_size in get_current_items(to_list(self.batch_size)):
-                assert isinstance(batch_size, int), "unsupported batch_size format: {}".format(type(batch_size))
+                assert isinstance(batch_size, (int, dict)), "unsupported batch_size format: {}".format(type(batch_size))
+                if isinstance(batch_size, dict):
+                    assert all([key in {"train", "eval", "test", "infer"} for key in batch_size.keys()]), \
+                        "batch size dictionaries must be keyed on mode"
+                    assert all([isinstance(val, int) for val in batch_size.values()]), \
+                        "batch size dictionary values must be integers"
             # ops check
             for op in get_current_items(self.ops):
                 assert isinstance(op, NumpyOp), "unsupported op format, must provide NumpyOp in Pipeline"
@@ -156,7 +161,12 @@ class Pipeline:
                     all_modes.append(mode)
         return to_set(all_modes)
 
-    def benchmark(self, mode: str = "train", epoch: int = 1, num_steps: int = 1000, log_interval: int = 100) -> None:
+    def benchmark(self,
+                  mode: str = "train",
+                  epoch: int = 1,
+                  num_steps: int = 1000,
+                  log_interval: int = 100,
+                  detailed: bool = True) -> None:
         """Benchmark the pipeline processing speed.
 
         Args:
@@ -164,6 +174,7 @@ class Pipeline:
             epoch: The epoch index to benchmark. Note that epoch indices are 1-indexed.
             num_steps: The maximum number of steps over which to perform the benchmark.
             log_interval: The logging interval.
+            detailed: Whether to display the detailed time used by each operator.
         """
         loader = self.get_loader(mode=mode, epoch=epoch)
         if isinstance(loader, tf.data.Dataset):
@@ -178,32 +189,36 @@ class Pipeline:
             if idx == num_steps:
                 break
         # Pipeline Operations Benchmarking when using FEDataset
-        if isinstance(loader, DataLoader) and isinstance(loader.dataset, OpDataset):
+        if isinstance(loader, DataLoader) and isinstance(loader.dataset, OpDataset) and detailed:
             op_list = loader.dataset.ops
             duration_list = np.zeros(shape=(len(op_list)))
 
             data_len = len(loader.dataset.dataset)
             if self.batch_size:
-                log_interval = log_interval * self.batch_size
+                batch_size = self.batch_size.get_current_value(epoch) if isinstance(self.batch_size,
+                                                                                    Scheduler) else self.batch_size
+                batch_size = batch_size[mode] if isinstance(batch_size, dict) else batch_size
+                log_interval = log_interval * batch_size
 
             print("\nBreakdown of time taken by Pipeline Operations ({} epoch {})".format(mode, epoch))
             for _ in range(log_interval):
                 index = np.random.randint(data_len)
                 items = deepcopy(loader.dataset.dataset[index])
                 if isinstance(loader.dataset.dataset, BatchDataset):
-                    unique_list = []
+                    # BatchDataset may randomly sample the same elements multiple times, so need to avoid reprocessing
+                    unique_samples = set()
                     for item in items:
-                        if id(item) not in unique_list:
+                        if id(item) not in unique_samples:
                             for i, op in enumerate(op_list):
                                 start = time.perf_counter()
-                                forward_numpyop([op], item, loader.dataset.mode)
+                                forward_numpyop([op], item, {'mode': loader.dataset.mode})
                                 duration = time.perf_counter() - start
                                 duration_list[i] += duration
-                            unique_list.append(id(item))
+                            unique_samples.add(id(item))
                 else:
                     for i, op in enumerate(op_list):
                         start = time.perf_counter()
-                        forward_numpyop([op], items, loader.dataset.mode)
+                        forward_numpyop([op], items, {'mode': loader.dataset.mode})
                         duration = time.perf_counter() - start
                         duration_list[i] += duration
 
@@ -276,7 +291,7 @@ class Pipeline:
         """
         data = deepcopy(data)
         ops = get_current_items(self.ops, mode, epoch)
-        forward_numpyop(ops, data, mode)
+        forward_numpyop(ops, data, {'mode': mode})
         for key, value in data.items():
             data[key] = np.expand_dims(value, 0)
         return data
@@ -306,8 +321,11 @@ class Pipeline:
             results = results[0]
         return results
 
-    def get_loader(self, mode: str, epoch: int = 1,
-                   shuffle: Optional[bool] = None) -> Union[DataLoader, tf.data.Dataset]:
+    def get_loader(self,
+                   mode: str,
+                   epoch: int = 1,
+                   shuffle: Optional[bool] = None,
+                   output_keys: Optional[Set[str]] = None) -> Union[DataLoader, tf.data.Dataset]:
         """Get a data loader from the Pipeline for a given `mode` and `epoch`.
 
         Args:
@@ -315,6 +333,7 @@ class Pipeline:
             epoch: The epoch index for the loader. Note that epoch indices are 1-indexed.
             shuffle: Whether to shuffle the data. If None, the value for shuffle is based on mode. NOTE: This argument
                 is only used with FastEstimator Datasets.
+            output_keys: What keys can be produced from pipeline. If None, all keys will be considered.
 
         Returns:
             A data loader for the given `mode` and `epoch`.
@@ -327,6 +346,8 @@ class Pipeline:
             batch_size = self.batch_size
             if isinstance(batch_size, Scheduler):
                 batch_size = batch_size.get_current_value(epoch)
+            if isinstance(batch_size, dict):
+                batch_size = batch_size[mode]
             # batch dataset
             if isinstance(data, BatchDataset):
                 data.pad_value = self.pad_value
@@ -337,13 +358,19 @@ class Pipeline:
             collate_fn = self.collate_fn
             if collate_fn is None and self.pad_value is not None:
                 collate_fn = self._pad_batch_collate
-            op_dataset = OpDataset(data, get_current_items(self.ops, mode, epoch), mode)
+            op_dataset = OpDataset(data,
+                                   get_current_items(self.ops, mode, epoch),
+                                   mode,
+                                   output_keys,
+                                   deep_remainder=False)
+            # Results will be immediately converted to tensors, so don't need deep_remainder
+            batch_size = None if isinstance(data, BatchDataset) else batch_size
             data = DataLoader(op_dataset,
-                              batch_size=None if isinstance(data, BatchDataset) else batch_size,
+                              batch_size=batch_size,
                               shuffle=False if isinstance(data, BatchDataset) else shuffle,
                               sampler=RandomSampler(op_dataset) if isinstance(data, BatchDataset) and shuffle else None,
                               num_workers=self.num_process,
-                              drop_last=self.drop_last,
+                              drop_last=False if batch_size is None else self.drop_last,
                               worker_init_fn=lambda _: np.random.seed(random.randint(0, 2**32 - 1)),
                               collate_fn=collate_fn)
         return data
