@@ -1,18 +1,5 @@
-# Copyright 2021 The FastEstimator Authors. All Rights Reserved.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-# ==============================================================================
-from typing import Any
+from collections import ChainMap
+from typing import Any, Dict
 
 import matplotlib.pyplot as plt
 import numpy as np
@@ -24,6 +11,8 @@ import fastestimator as fe
 from fastestimator.dataset.numpy_dataset import NumpyDataset
 from fastestimator.op.tensorop.model import ModelOp
 from fastestimator.summary import System
+from fastestimator.trace import Trace
+from fastestimator.util.data import Data
 
 
 def is_equal(obj1: Any, obj2: Any, assert_type: bool = True, assert_dtype: bool = False) -> bool:
@@ -251,3 +240,45 @@ def fig_to_rgb_array(fig: plt.Figure) -> np.ndarray:
     buf = fig.canvas.tostring_rgb()
     ncols, nrows = fig.canvas.get_width_height()
     return np.frombuffer(buf, dtype=np.uint8).reshape(nrows, ncols, 3)
+
+
+class TraceRun:
+    """Class to simulate the trace calling protocol.
+
+    This serve for testing purpose without using estimator class.
+
+    Args:
+        trace: Target trace to run.
+        batch: Batch data from pipepline.
+        prediction: Batch data from network.
+    """
+    def __init__(self, trace: Trace, batch: Dict[str, Any], prediction: Dict[str, Any]):
+        self.trace = trace
+        self.batch = batch
+        self.prediction = prediction
+        self.data_on_begin = None
+        self.data_on_end = None
+        self.data_on_epoch_begin = None
+        self.data_on_epoch_end = None
+        self.data_on_batch_begin = None
+        self.data_on_batch_end = None
+
+
+    def run_trace(self) -> None:
+        self.data_on_begin = Data()
+        self.trace.on_begin(self.data_on_begin)
+
+        self.data_on_epoch_begin = Data()
+        self.trace.on_epoch_begin(self.data_on_epoch_begin)
+
+        self.data_on_batch_begin = Data(self.batch)
+        self.trace.on_batch_begin(self.data_on_batch_begin)
+
+        self.data_on_batch_end = Data(ChainMap(self.prediction, self.batch))
+        self.trace.on_batch_end(self.data_on_batch_end)
+
+        self.data_on_epoch_end = Data()
+        self.trace.on_epoch_end(self.data_on_epoch_end)
+
+        self.data_on_end = Data()
+        self.trace.on_end(self.data_on_end)
