@@ -139,14 +139,16 @@ class FeSplitSummary(LatexObject):
         super().__init__()
         self.data = []
 
-    def add_split(self, parent: Union[FEID, str], fraction: str) -> None:
+    def add_split(self, parent: Union[FEID, str], fraction: str, seed: Optional[int], stratify: Optional[str]) -> None:
         """Record another split on this dataset.
 
         Args:
             parent: The id of the parent involved in the split (or 'self' if you are the parent).
             fraction: The string representation of the split fraction that was used.
+            seed: The random seed used during the split.
+            stratify: The stratify key used during the split.
         """
-        self.data.append((parent, fraction))
+        self.data.append((parent, fraction, seed, stratify))
 
     def dumps(self) -> str:
         """Generate a LaTeX formatted representation of this object.
@@ -155,9 +157,12 @@ class FeSplitSummary(LatexObject):
             A LaTeX string representation of this object.
         """
         return " $\\rightarrow$ ".join([
-            f"{HrefFEID(parent, name='').dumps() if isinstance(parent, FEID) else parent}({escape_latex(fraction)})"
-            for parent,
-            fraction in self.data
+            f"{HrefFEID(parent, name='').dumps() if isinstance(parent, FEID) else parent}({escape_latex(fraction)}" +
+            (f", seed={seed}" if seed is not None else "") +
+            (f", stratify=`{escape_latex(stratify)}'" if stratify is not None else "") + ")" for parent,
+            fraction,
+            seed,
+            stratify in self.data
         ])
 
 
@@ -1007,6 +1012,13 @@ def __getstate__(self) -> Dict[str, Any]:
         The state variables to be captured during pickling.
     """
     state_dict = self.__dict__.copy()
+    # Classes which subclass a @traceable class but are not themselves marked as @traceable will not inherit their
+    # parent whitelist and blacklist. If __getstate__ is being invoked then they'll need at least the basics.
+    if not hasattr(self, '_fe_state_whitelist'):
+        self._fe_state_whitelist = ()
+    if not hasattr(self, '_fe_state_blacklist'):
+        self._fe_state_blacklist = ('_fe_state_whitelist', '_fe_state_blacklist', '_fe_traceability_summary')
+    # Apply the whitelist and blacklist
     if self._fe_state_whitelist:
         state_dict = {key: state_dict[key] for key in self._fe_state_whitelist}
     for key in self._fe_state_blacklist:
@@ -1057,6 +1069,7 @@ def _setdata(current: Any, new: Any) -> Any:
     if isinstance(current, tuple) and isinstance(new, (tuple, list)) and len(current) == len(new):
         return tuple([_setdata(cu, cr) for cu, cr in zip(current, new)])
     if isinstance(current, dict) and isinstance(new, dict) and current.keys() == new.keys():
+        # Might want to consider removing the key equality requirement, but unclear what ramifications that would have.
         for key in current.keys():
             current[key] = _setdata(current[key], new[key])
         return current
