@@ -14,6 +14,7 @@
 # ==============================================================================
 """Utilities for FastEstimator."""
 import json
+import os
 import re
 import string
 import sys
@@ -184,6 +185,34 @@ def to_set(data: Any) -> Set[Any]:
     return data
 
 
+def param_to_range(
+        data: Union[int, float, Tuple[int, int], Tuple[float, float]]) -> Union[Tuple[int, int], Tuple[float, float]]:
+    """Convert a single int or float value to a tuple signifying a range.
+
+    ```python
+    x = fe.util.param_to_tuple(7)  # (-7, 7)
+    x = fe.util.param_to_tuple([7, 8])  # (7,8))
+    x = fe.util.param_to_tuple((3.1, 4.3))  # (3.1, 4.3)
+    x = fe.util.to_set((-3.2))  # (-3.2, 3.2)
+    ```
+
+    Args:
+        data: Input data.
+
+    Returns:
+        The input `data` but in tuple form for a range.
+    """
+    if isinstance(data, (int, float)):
+        if data > 0:
+            data = -data, data
+        else:
+            data = data, -data
+    elif isinstance(data, (list, tuple)):
+        data = tuple(data)
+
+    return data
+
+
 class NonContext(object):
     """A class which is used to make nothing unusual happen.
 
@@ -217,16 +246,22 @@ class Suppressor(object):
     ```
     """
     def __enter__(self) -> None:
-        # pylint: disable=attribute-defined-outside-init
-        self.stdout = sys.stdout
-        self.stderr = sys.stderr
-        # pylint: enable=attribute-defined-outside-init
-        sys.stdout = self
-        sys.stderr = self
+        # This is not necessary to block printing, but lets the system know what's happening
+        self.py_reals = [sys.stdout, sys.stderr]
+        sys.stdout = sys.stderr = self
+        # This part does the heavy lifting
+        self.fakes = [os.open(os.devnull, os.O_RDWR), os.open(os.devnull, os.O_RDWR)]
+        self.reals = [os.dup(1), os.dup(2)]  # [stdout, stderr]
+        os.dup2(self.fakes[0], 1)
+        os.dup2(self.fakes[1], 2)
 
     def __exit__(self, *exc: Tuple[Optional[Type], Optional[Exception], Optional[Any]]) -> None:
-        sys.stdout = self.stdout
-        sys.stderr = self.stderr
+        os.dup2(self.reals[0], 1)
+        os.dup2(self.reals[1], 2)
+        for fd in self.fakes + self.reals:
+            os.close(fd)
+        # Set the python pointers back too
+        sys.stdout, sys.stderr = self.py_reals[0], self.py_reals[1]
 
     def write(self, dummy: str) -> None:
         """A function which is invoked during print calls.
@@ -236,11 +271,6 @@ class Suppressor(object):
         """
         pass
 
-    def flush(self) -> None:
-        """A function to empty the current print buffer. No-op in this case.
-        """
-        pass
-
 
 class LogSplicer:
     """A class to send stdout information into a file before passing it along to the normal stdout.
@@ -663,10 +693,10 @@ def show_image(im: Union[np.ndarray, Tensor],
         color = ["m", "r", "c", "g", "y", "b"][stack_depth % 6]
         for box in im:
             # Unpack the box, which may or may not have a label
-            x0 = int(box[0])
-            y0 = int(box[1])
-            width = int(box[2])
-            height = int(box[3])
+            x0 = float(box[0])
+            y0 = float(box[1])
+            width = float(box[2])
+            height = float(box[3])
             label = None if len(box) < 5 else str(box[4])
 
             # Don't draw empty boxes, or invalid box
