@@ -16,7 +16,6 @@ import os
 import pickle
 import unittest
 import urllib.request
-import warnings
 
 
 def ping(host):
@@ -26,9 +25,8 @@ def ping(host):
         req.add_header('User-Agent', agent)
         code = urllib.request.urlopen(req).getcode()
         return code == 200
-    except:
+    except urllib.request.HTTPError as e:
         return False
-    return False
 
 
 class TestData(unittest.TestCase):
@@ -39,18 +37,11 @@ class TestData(unittest.TestCase):
             cls.data_urls = pickle.load(dataset_url_dict)
 
     def test_dataset_urls(self):
-        failed_urls = set()
         for key, value in self.data_urls.items():
             if isinstance(value, list):
                 for url in value:
                     with self.subTest('{}{} url'.format(key, url)):
-                        if not ping(url):
-                            failed_urls.add(key)
-                            print("\033[93m {} is not reachable at {}\033[00m".format(key, url))
+                        self.assertTrue(ping(url))
             else:
                 with self.subTest('Check if {} url reachable'.format(key)):
-                    if not ping(value):
-                        failed_urls.add(key)
-                        print("\033[93m {} is not reachable at {}\033[00m".format(key, value))
-        if failed_urls:
-            self.skipTest("The following datasets were not available: {}".format(", ".join(failed_urls)))
+                    self.assertTrue(ping(value))
